## This R code accompanies the publication: "Potential parasite transmission in multi-host networks based on parasite 
## sharing" by Pilosof, Morand, Krasnov and Nunn (2014) in PloS One. Note that some analyses are done with software
## external to R (bipartmod).

## Note that the code is best viewed with R-Studio (free donwload in http://www.rstudio.com/)


################################# LOAD LIBRARIES -----------------------
library(bipartite)
library(igraph)
library(vegan)
library(gdata)
library(XML)
library(ape)
library(nlme)
library(MASS)
library(MuMIn)
library(stringr)
library(Hmisc)
library(ecodist)
library(rgexf)
library(corrgram)
library(ggplot2)
library(gridExtra)
library(plyr)
library(RColorBrewer)


################################# Load functions ------------------------------------------------------------
# Connectance
connectance <- function(M, onemode=F){
  if (onemode == T) {return((sum(M > 0)/(dim(M)[1]*dim(M)[2]))/2)}
  else return(sum(M > 0)/(dim(M)[1]*dim(M)[2]))
}


# The probabilistic model from Fortuna et al. 2010, J Anim Ecol
# This version takes care first to fill every row with at least 1 value
prob_model <- function (A, iter=1000, verbose=T) {
  rows=dim(A)[[1]]
  columns=dim(A)[[2]]
  prob_row <- rowSums(A)/columns # calculate intrarow probabilities 
  prob_column <- colSums(A)/rows # calculate intracolumn probabilities 
  
  rep=0
  attempt <- 1
  null.matrices <- list()
  while (rep < iter){    # number of random matrices
    B <- A; B[,] <- 0 # Create a new matrix to be filled with 1's
    for (i in 1:rows){  
      # First make sure that at least one column is 1 for row i
      row.ok=F
      j=1
      while (row.ok == F){
        if (runif(1,0,1) <= (prob_row[i]+prob_column[j])/2) { # randomization of each cell is the average of row and column probabilities
          B[i,j] <- 1
          row.ok=T
        }
        j <- j+1
        if (j==columns){j=1}
      }
      # Then fill in the rest of the columns
      for (j in 1:columns){
        if (runif(1,0,1) <= (prob_row[i]+prob_column[j])/2) { # randomization of each cell is the average of row and column probabilities
          B[i,j] <- 1
        }
      }
    }
    if (sum(rowSums(B) == 0) == 0 & sum(colSums(B) == 0) == 0){ # all rows and columns have at least one link each?
      rep <- rep+1 # Randomize another matrix
      null.matrices[[rep]] <- B
      attempt <- 1
    }
    else {
      if (verbose) {print(paste('Iter: ',rep, ' One or more rows/cols are missing interactions, attempt #',attempt,sep=''))}
      attempt <- attempt+1
    }
  }
  return(null.matrices)
}

# A function to create an empty data frame. Found it at: https://stat.ethz.ch/pipermail/r-help/2008-August/169968.html
empty.df<- function(header){
  df <- data.frame(matrix(matrix(rep(1,length(header)),1),1))
  colnames(df )<- header
  return(df[NULL,])
}


TPN <- function(web.bip, attrib=NULL){
  web.list <- list()
  web.list[['bipartite']] <- web.bip
  web.list[['C.bip']] <- connectance(web.bip, onemode=F);
  # Project the network: edge weight is number of shared parasites
  web.uni <- as.one.mode(web.bip, project='lower')
  web.uni <- bipartite::empty(web.uni)
  web.list[['unip.Sum']] <- web.uni
  web.list[['C.uni']] <- connectance(web.uni, onemode=T);
  # Project the network: edge weight is beta diversity with the Sorenson index (Beta_sor in Koleff et al. 2003)
  # Shared parasites:
  web.bdiv <- as.matrix(betadiver(web.bip, method = 'j'))
  web.bdiv <- bipartite::empty(web.bdiv)
  web.list[['unip.betaD']] <- web.bdiv
  
  g <- graph.adjacency(web.bdiv, mode='max', weighted=T)
  # Summary of link weights
  web.list[['unipartite.link.weights']] <- round(table(E(g)$weight)/sum(table(E(g)$weight))*100,1)
  web.list[['unipartite.degree']] <- degree(g)
  # Initialize the web
  V(g)$ID <- 1:length(V(g))
  V(g)$Species <- as.character(attrib$RodentSp[match(V(g)$name,attrib$RodentID)])
  #V(g)$color <- as.character(attrib$ColorCode[match(V(g)$name,attrib$RodentID)])
  V(g)$label <- paste(V(g)$ID, V(g)$name,sep='_')
  EC <- evcent(g, weights=NULL)
  EC <- EC$vector
  attrib$EC <- EC[match(attrib$RodentID, names(EC))]
  V(g)$EC <- EC[match(V(g)$name, names(EC))]
  web.list[['g']] <- g
  web.list[['attrib']] <- attrib
  return(web.list)
}


# StepAICc
# Downloaded 2013-7-30 from Christoph Scherber's website
# http://wwwuser.gwdg.de/~cscherb1/content/PDF%20Files/StepAICc%20for%20lm%20glm%20and%20lme%20models.pdf


# StepAICc for linear models
#
# (1) extractAICc
# (2) dropterm.AICc
# (3) addterm.AICc
# (4) stepAICc
#
# Code originally written by B.D. Ripley and W.N. Venables
# with changes in the extractAICc function and others written
# by C. Scherber, 4th/5th May 2009. Adjusted to work for
# lm, glm and lme models on 8th October 2011.
#
# The function stepAICc may be used to perform AICc-based model selection
# if sample sizes are too small to use AIC.
#
# AICc is a version of AIC corrected for small sample sizes
# as defined in Burnham & Anderson, 2002.
extractAICc=function (fit, scale, k = 2, ...)
{
  if ( any(class(fit)=="lme")==T)
    if (fit$method != "ML")
      stop("AIC undefined for REML fit")
  res <- logLik(fit)
  edf <- attr(res, "df")
  n=length(residuals(fit))
  c(edf,-2 * res + k * edf+2*edf*(edf+1)/(n-edf-1))
}
dropterm.AICc=
  function (object, scope, scale = 0, test = c("none", "Chisq"),
            k = 2, sorted = FALSE, trace = FALSE, ...)
  {
    tl <- attr(terms(object), "term.labels")
    if (missing(scope))
      scope <- drop.scope(object)
    else {
      if (!is.character(scope))
        scope <- attr(terms(update.formula(object, scope)),
                      "term.labels")
      if (!all(match(scope, tl, 0L)))
        stop("scope is not a subset of term labels")
    }
    ns <- length(scope)
    ans <- matrix(nrow = ns + 1L, ncol = 2L, dimnames = list(c("<none>",
                                                               scope), c("df", "AIC")))
    ans[1, ] <- extractAICc(object, scale, k = k, ...)
    env <- environment(formula(object))
    n0 <- length(object$residuals)
    for (i in seq(ns)) {
      tt <- scope[i]
      if (trace) {
        message("trying -", tt)
        utils::flush.console()
      }
      nfit <- update(object, as.formula(paste("~ . -", tt)),
                     evaluate = FALSE)
      nfit <- eval(nfit, envir = env)
      ans[i + 1, ] <- extractAICc(nfit, scale, k = k, ...)
      if (length(nfit$residuals) != n0)
        stop("number of rows in use has changed: remove missing values?")
    }
    dfs <- ans[1L, 1L] - ans[, 1L]
    dfs[1L] <- NA
    aod <- data.frame(Df = dfs, AIC = ans[, 2])
    o <- if (sorted)
      order(aod$AIC)
    else seq_along(aod$AIC)
    test <- match.arg(test)
    if (test == "Chisq") {
      dev <- ans[, 2L] - k * ans[, 1L]
      dev <- dev - dev[1L]
      dev[1L] <- NA
      nas <- !is.na(dev)
      P <- dev
      P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
      aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
    }
    aod <- aod[o, ]
    head <- c("Single term deletions", "\nModel:",
              deparse(as.vector(formula(object))))
    if (scale > 0)
      head <- c(head, paste("\nscale: ", format(scale), "\n"))
    class(aod) <- c("anova", "data.frame")
    attr(aod, "heading") <- head
    aod
  }
##
addterm.AICc=function (object, scope, scale = 0, test = c("none", "Chisq"),
                       k = 2, sorted = FALSE, trace = FALSE, ...)
{
  if (missing(scope) || is.null(scope))
    stop("no terms in scope")
  if (!is.character(scope))
    scope <- add.scope(object, update.formula(object, scope))
  if (!length(scope))
    stop("no terms in scope for adding to object")
  ns <- length(scope)
  ans <- matrix(nrow = ns + 1L, ncol = 2L, dimnames = list(c("<none>",
                                                             scope), c("df", "AIC")))
  ans[1L, ] <- extractAICc(object, scale, k = k, ...)
  n0 <- length(object$residuals)
  env <- environment(formula(object))
  for (i in seq(ns)) {
    tt <- scope[i]
    if (trace) {
      message("trying +", tt)
      utils::flush.console()
    }
    nfit <- update(object, as.formula(paste("~ . +", tt)),
                   evaluate = FALSE)
    nfit <- eval(nfit, envir = env)
    ans[i + 1L, ] <- extractAICc(nfit, scale, k = k, ...)
    if (length(nfit$residuals) != n0)
      stop("number of rows in use has changed: remove missing values?")
  }
  dfs <- ans[, 1L] - ans[1L, 1L]
  dfs[1L] <- NA
  aod <- data.frame(Df = dfs, AIC = ans[, 2L])
  o <- if (sorted)
    order(aod$AIC)
  else seq_along(aod$AIC)
  test <- match.arg(test)
  if (test == "Chisq") {
    dev <- ans[, 2L] - k * ans[, 1L]
    dev <- dev[1L] - dev
    dev[1L] <- NA
    nas <- !is.na(dev)
    P <- dev
    P[nas] <- safe_pchisq(dev[nas], dfs[nas], lower.tail = FALSE)
    aod[, c("LRT", "Pr(Chi)")] <- list(dev, P)
  }
  aod <- aod[o, ]
  head <- c("Single term additions", "\nModel:",
            deparse(as.vector(formula(object))))
  if (scale > 0)
    head <- c(head, paste("\nscale: ", format(scale), "\n"))
  class(aod) <- c("anova", "data.frame")
  attr(aod, "heading") <- head
  aod
}
##
stepAICc=function (object, scope, scale = 0, direction = c("both",
                                                           "backward",
                                                           "forward"), trace = 1, keep = NULL, steps = 1000, use.start = FALSE,
                   k = 2, ...)
{
  mydeviance <- function(x, ...) {
    dev <- deviance(x)
    if (!is.null(dev))
      dev
    else extractAICc(x, k = 0)[2L]
  }
  cut.string <- function(string) {
    if (length(string) > 1L)
      string[-1L] <- paste("\n", string[-1L], sep = "")
    string
  }
  re.arrange <- function(keep) {
    namr <- names(k1 <- keep[[1L]])
    namc <- names(keep)
    nc <- length(keep)
    nr <- length(k1)
    array(unlist(keep, recursive = FALSE), c(nr, nc), list(namr,
                                                           namc))
  }
  step.results <- function(models, fit, object, usingCp = FALSE) {
    change <- sapply(models, "[[", "change")
    rd <- sapply(models, "[[", "deviance")
    dd <- c(NA, abs(diff(rd)))
    rdf <- sapply(models, "[[", "df.resid")
    ddf <- c(NA, abs(diff(rdf)))
    AIC <- sapply(models, "[[", "AIC")
    heading <- c("Stepwise Model Path \nAnalysis of Deviance Table",
                 "\nInitial Model:", deparse(as.vector(formula(object))),
                 "\nFinal Model:", deparse(as.vector(formula(fit))),
                 "\n")
    aod <- if (usingCp)
      data.frame(Step = change, Df = ddf, Deviance = dd,
                 `Resid. Df` = rdf, `Resid. Dev` = rd, Cp = AIC,
                 check.names = FALSE)
    else data.frame(Step = change, Df = ddf, Deviance = dd,
                    `Resid. Df` = rdf, `Resid. Dev` = rd, AIC = AIC,
                    check.names = FALSE)
    attr(aod, "heading") <- heading
    class(aod) <- c("Anova", "data.frame")
    fit$anova <- aod
    fit
  }
  Terms <- terms(object)
  object$formula <- Terms
  if (inherits(object, "lme"))
    object$call$fixed <- Terms
  else if (inherits(object, "gls"))
    object$call$model <- Terms
  else object$call$formula <- Terms
  if (use.start)
    warning("'use.start' cannot be used with R's version of glm")
  md <- missing(direction)
  direction <- match.arg(direction)
  backward <- direction == "both" | direction == "backward"
  forward <- direction == "both" | direction == "forward"
  if (missing(scope)) {
    fdrop <- numeric(0)
    fadd <- attr(Terms, "factors")
    if (md)
      forward <- FALSE
  }
  else {
    if (is.list(scope)) {
      fdrop <- if (!is.null(fdrop <- scope$lower))
        attr(terms(update.formula(object, fdrop)), "factors")
      else numeric(0)
      fadd <- if (!is.null(fadd <- scope$upper))
        attr(terms(update.formula(object, fadd)), "factors")
    }
    else {
      fadd <- if (!is.null(fadd <- scope))
        attr(terms(update.formula(object, scope)), "factors")
      fdrop <- numeric(0)
    }}
  models <- vector("list", steps)
  if (!is.null(keep))
    keep.list <- vector("list", steps)
  if (is.list(object) && (nmm <- match("nobs", names(object),
                                       0)) > 0)
    n <- object[[nmm]]
  else n <- length(residuals(object))
  fit <- object
  bAIC <- extractAICc(fit, scale, k = k, ...)
  edf <- bAIC[1L]
  bAIC <- bAIC[2L]
  if (is.na(bAIC))
    stop("AIC is not defined for this model, so stepAIC cannot proceed")
  nm <- 1
  Terms <- terms(fit)
  if (trace) {
    cat("Start: AICc=", format(round(bAIC, 2)), "\n",
        cut.string(deparse(as.vector(formula(fit)))),
        "\n\n", sep = "")
    utils::flush.console()
  }
  models[[nm]] <- list(deviance = mydeviance(fit), df.resid = n -
                         edf, change = "", AIC = bAIC)
  if (!is.null(keep))
    keep.list[[nm]] <- keep(fit, bAIC)
  usingCp <- FALSE
  while (steps > 0) {
    steps <- steps - 1
    AIC <- bAIC
    ffac <- attr(Terms, "factors")
    if (!is.null(sp <- attr(Terms, "specials")) && !is.null(st <- sp$strata))
      ffac <- ffac[-st, ]
    scope <- factor.scope(ffac, list(add = fadd, drop = fdrop))
    aod <- NULL
    change <- NULL
    if (backward && length(scope$drop)) {
      aod <- dropterm.AICc(fit, scope$drop, scale = scale, trace = max(0,
                                                                       trace - 1), k = k, ...)
      rn <- row.names(aod)
      row.names(aod) <- c(rn[1L], paste("-", rn[-1L], sep = " "))
      if (any(aod$Df == 0, na.rm = TRUE)) {
        zdf <- aod$Df == 0 & !is.na(aod$Df)
        nc <- match(c("Cp", "AIC"), names(aod))
        nc <- nc[!is.na(nc)][1L]
        ch <- abs(aod[zdf, nc] - aod[1, nc]) > 0.01
        if (any(ch)) {
          warning("0 df terms are changing AIC")
          zdf <- zdf[!ch]
        }
        if (length(zdf) > 0L)
          change <- rev(rownames(aod)[zdf])[1L]
      }}
    if (is.null(change)) {
      if (forward && length(scope$add)) {
        aodf <- addterm.AICc(fit, scope$add, scale = scale,
                             trace = max(0, trace - 1), k = k, ...)
        rn <- row.names(aodf)
        row.names(aodf) <- c(rn[1L], paste("+", rn[-1L],
                                           sep = " "))
        aod <- if (is.null(aod))
          aodf
        else rbind(aod, aodf[-1, , drop = FALSE])
      }
      attr(aod, "heading") <- NULL
      if (is.null(aod) || ncol(aod) == 0)
        break
      nzdf <- if (!is.null(aod$Df))
        aod$Df != 0 | is.na(aod$Df)
      aod <- aod[nzdf, ]
      if (is.null(aod) || ncol(aod) == 0)
        break
      nc <- match(c("Cp", "AIC"), names(aod))
      nc <- nc[!is.na(nc)][1L]
      o <- order(aod[, nc])
      if (trace) {
        print(aod[o, ])
        utils::flush.console()
      }
      if (o[1L] == 1)
        break
      change <- rownames(aod)[o[1L]]
    }
    usingCp <- match("Cp", names(aod), 0) > 0
    fit <- update(fit, paste("~ .", change), evaluate = FALSE)
    fit <- eval.parent(fit)
    if (is.list(fit) && (nmm <- match("nobs", names(fit),
                                      0)) > 0)
      nnew <- fit[[nmm]]
    else nnew <- length(residuals(fit))
    if (nnew != n)
      stop("number of rows in use has changed: remove missing values?")
    Terms <- terms(fit)
    bAIC <- extractAICc(fit, scale, k = k, ...)
    edf <- bAIC[1L]
    bAIC <- bAIC[2L]
    if (trace) {
      cat("\nStep: AICc=", format(round(bAIC, 2)), "\n",
          cut.string(deparse(as.vector(formula(fit)))),
          "\n\n", sep = "")
      utils::flush.console()
    }
    if (bAIC >= AIC + 1e-07)
      break
    nm <- nm + 1
    models[[nm]] <- list(deviance = mydeviance(fit), df.resid = n -
                           edf, change = change, AIC = bAIC)
    if (!is.null(keep))
      keep.list[[nm]] <- keep(fit, bAIC)
  }
  if (!is.null(keep))
    fit$keep <- re.arrange(keep.list[seq(nm)])
  step.results(models = models[seq(nm)], fit, object, usingCp)
}

# A function that tests and plots normality assumptions of linear models
normality.assumptions <- function (model) {
  layout(matrix(1:3,3,1))
  #get unstandardized predicted and residual values
  unstandardizedPredicted <- predict(model)
  unstandardizedResiduals <- resid(model)
  standardizedPredicted <- (unstandardizedPredicted - mean(unstandardizedPredicted)) / sd(unstandardizedPredicted)
  standardizedResiduals <- (unstandardizedResiduals - mean(unstandardizedResiduals)) / sd(unstandardizedResiduals)
  #create standardized residuals plot
  plot(standardizedPredicted, standardizedResiduals, main = "Standardized Residuals Plot", xlab = "Standardized Predicted Values", ylab = "Standardized Residuals")
  #add horizontal line
  abline(0,0)
  #create residuals histogram
  hist(standardizedResiduals, freq = FALSE)
  #add normal curve
  curve(dnorm, add = TRUE)
  #get probability distribution for residuals
  probDist <- pnorm(standardizedResiduals)
  #create PP plot
  plot(ppoints(length(standardizedResiduals)), sort(probDist), main = "PP Plot", xlab = "Observed Probability", ylab = "Expected Probability")
  #add diagonal line
  abline(0,1)
  print(shapiro.test(resid(model)))
}

MRMlogit <- function(formula = formula(data), data = sys.parent(), nperm = 1000)
{
  m <- match.call(expand.dots = FALSE)
  m2 <- match(c("formula", "data"), names(m), nomatch = 0)
  m <- m[c(1, m2)]
  m[[1]] <- as.name("model.frame")
  m <- eval(m, parent.frame())
  m <- as.matrix(m)
  n <- (1 + sqrt(1 + 8 * nrow(m)))/2
  if (abs(n - round(n)) > 1e-07) 
    stop("Matrix not square.\n")
  n <- round(n)
  if (ncol(m) < 2) 
    stop("Not enough data. \n")
  for (thiscol in 1:ncol(m)) {
    tempmat <- full(m[, thiscol])
    m[, thiscol] <- tempmat[col(tempmat) > row(tempmat)]
  }
  X <- m[, 2:ncol(m), drop = FALSE]
  Y <- m[, 1, drop = FALSE]
  colnames(Y) <- "Y"
  
  newdata <- data.frame(Y=Y, X)
  fit1 <- glm(Y ~ ., data=newdata, family=binomial(link = "logit"))
  
  # want to save coefficients, deviance & df
  b <- coefficients(fit1)
  dev <- summary(fit1)$deviance
  dev.df <- summary(fit1)$df.residual
  
  if(nperm > 0) {
    b.all <- matrix(NA, nrow=nperm, ncol=length(b))
    b.all[1,] <- b
    dev.all <- rep(NA, nperm)
    dev.all[1] <- dev
    
    for(i in 2:nperm) {
      newSample <- sample(n)
      newY <- full(Y)
      newY <- newY[newSample, newSample]
      newY <- lower(newY)
      newdata <- data.frame(Y=newY, X=X)
      newfit <- glm(Y ~ ., data=newdata, family=binomial(link = "logit"))
      b.all[i,] <- coefficients(newfit)
      dev.all[i] <- summary(newfit)$deviance
    }
    b.pval <- apply(b.all, 2, function(x) length(x[abs(x) >= abs(x[1])])/nperm)
    dev.pval <- length(dev.all[dev.all >= dev.all[1]])/nperm
    
  }
  list(coef = cbind(b, pval = b.pval), dev = c(resid.dev = dev, resid.df = dev.df, dev.pval = dev.pval))
  
}


################################# Organize data -------------------
setwd (MyWorkingFolder)
# Load the node attributes
A <- read.csv('Attrib_Indiv_Helminths_publication.csv', he=T)
A$RodentID <- as.character(A$RodentID)
A$RodentSp <- factor(A$RodentSp)
A$Age <- factor(A$Age)
# Load the matrix data for networks
M <- read.csv('M_Indiv_Helminths_publication.csv', he=T)
# Transform the data frame to matrix
rown <- M[,1]
M <- data.matrix(M[2:length(M)])
dimnames(M)[[1]] <- rown

# Data for Mondolkiri
A.Mon <- A[A$Locality == 'MON',]
A.Mon <- A.Mon[-which(A.Mon$RodentSp %in% names(which(table(A.Mon$RodentSp) < 5))),] # Include species with 5 or more indiv.
A.Mon$RodentSp <- factor(A.Mon$RodentSp)
A.Mon$NodeID <- 1:length(A.Mon$RodentID)
M.Mon <- M[rownames(M) %in% A.Mon$RodentID,]
M.Mon <- bipartite::empty(M.Mon)
table(A.Mon$RodentSp)

# Data for Sihanouk
A.Sih <- A[A$Locality == 'SIH',]
A.Sih <- A.Sih[-which(A.Sih$RodentSp %in% names(which(table(A.Sih$RodentSp) < 5))),]
A.Sih$RodentSp <- factor(A.Sih$RodentSp)
A.Sih$NodeID <- 1:length(A.Sih$RodentID)
M.Sih <- M[rownames(M) %in% A.Sih$RodentID,]
M.Sih <- bipartite::empty(M.Sih)
table(A.Sih$RodentSp)

# Data for Buriram
A.Bur <- A[A$Locality == 'BUR',]
A.Bur <- A.Bur[-which(A.Bur$RodentSp %in% names(which(table(A.Bur$RodentSp) < 5))),]
A.Bur$RodentSp <- factor(A.Bur$RodentSp)
A.Bur$NodeID <- 1:length(A.Bur$RodentID)
M.Bur <- M[rownames(M) %in% A.Bur$RodentID,]
M.Bur <- bipartite::empty(M.Bur)
table(A.Bur$RodentSp)

# total number of pathogens
sort(unique(c(colnames(M.Mon), colnames(M.Sih), colnames(M.Bur))))
# Shared between Localities
intersect(intersect(colnames(M.Mon), colnames(M.Sih)), colnames(M.Bur))

## Build single-species networks
A.Mon.Bs <- A.Mon[A.Mon$RodentSp == 'Bandicota_savilei',]
M.Mon.Bs <- M.Mon[rownames(M.Mon) %in% A.Mon.Bs$RodentID,]
M.Mon.Bs <- bipartite::empty(M.Mon.Bs)
# X11();plotweb(M.Mon.Bs)
A.Mon.Rt <- A.Mon[A.Mon$RodentSp == 'Rattus_tanezumi',]
M.Mon.Rt <- M.Mon[rownames(M.Mon) %in% A.Mon.Rt$RodentID,]
M.Mon.Rt <- bipartite::empty(M.Mon.Rt)
# X11();plotweb(M.Mon.Rt)
A.Sih.Ra <- A.Sih[A.Sih$RodentSp == 'Rattus_argentiventer',]
M.Sih.Ra <- M.Sih[rownames(M.Sih) %in% A.Sih.Ra$RodentID,]
M.Sih.Ra <- bipartite::empty(M.Sih.Ra)
# X11();plotweb(M.Sih.Ra)
A.Sih.Re <- A.Sih[A.Sih$RodentSp == 'Rattus_exulans',]
M.Sih.Re <- M.Sih[rownames(M.Sih) %in% A.Sih.Re$RodentID,]
M.Sih.Re <- bipartite::empty(M.Sih.Re)
# X11();plotweb(M.Sih.Re)
A.Sih.Rn <- A.Sih[A.Sih$RodentSp == 'Rattus_norvegicus',]
M.Sih.Rn <- M.Sih[rownames(M.Sih) %in% A.Sih.Rn$RodentID,]
M.Sih.Rn <- bipartite::empty(M.Sih.Rn)
# X11();plotweb(M.Sih.Rn)
A.Sih.Rt <- A.Sih[A.Sih$RodentSp == 'Rattus_tanezumi',]
M.Sih.Rt <- M.Sih[rownames(M.Sih) %in% A.Sih.Rt$RodentID,]
M.Sih.Rt <- bipartite::empty(M.Sih.Rt)
# X11();plotweb(M.Sih.Rt)
A.Bur.Bs <- A.Bur[A.Bur$RodentSp == 'Bandicota_savilei',]
M.Bur.Bs <- M.Bur[rownames(M.Bur) %in% A.Bur.Bs$RodentID,]
M.Bur.Bs <- bipartite::empty(M.Bur.Bs)
# X11();plotweb(M.Bur.Bs)
A.Bur.Mc <- A.Bur[A.Bur$RodentSp == 'Mus_cervicolor',]
M.Bur.Mc <- M.Bur[rownames(M.Bur) %in% A.Bur.Mc$RodentID,]
M.Bur.Mc <- bipartite::empty(M.Bur.Mc)
# X11();plotweb(M.Bur.Mc)
A.Bur.Re <- A.Bur[A.Bur$RodentSp == 'Rattus_exulans',]
M.Bur.Re <- M.Bur[rownames(M.Bur) %in% A.Bur.Re$RodentID,]
M.Bur.Re <- bipartite::empty(M.Bur.Re)
# X11();plotweb(M.Bur.Re)


################################# Make TPNs -------------------

# These lines make a Ttransmission Potential Network for each multi/single-species network.
web.Bur <- TPN(M.Bur, A.Bur)
web.Bur.Bs <- TPN(M.Bur.Bs, A.Bur.Bs)
web.Bur.Mc <- TPN(M.Bur.Mc, A.Bur.Mc)
web.Bur.Re <- TPN(M.Bur.Re, A.Bur.Re)
web.Mon <- TPN(M.Mon, A.Mon)
web.Mon.Bs <- TPN(M.Mon.Bs, A.Mon.Bs)
web.Mon.Rt <- TPN(M.Mon.Rt, A.Mon.Rt)
web.Sih <- TPN(M.Sih, A.Sih)
web.Sih.Ra <- TPN(M.Sih.Ra, A.Sih.Ra)
web.Sih.Re <- TPN(M.Sih.Re, A.Sih.Re)
web.Sih.Rn <- TPN(M.Sih.Rn, A.Sih.Rn)
web.Sih.Rt <- TPN(M.Sih.Rt, A.Sih.Rt)


################################# Phylogenetic tree  -------------------
setwd(MyWorkingFolder)
MyTree <- read.tree("tree.txt")
plot(MyTree)
# Leave only necessary tree tips
tips.to.drop <- MyTree$tip.label[!(MyTree$tip.label %in% unique(c(levels(A.Mon$RodentSp), levels(A.Sih$RodentSp), levels(A.Bur$RodentSp))))]
MyTree <- drop.tip(MyTree, tips.to.drop)
plot(MyTree)


################################# Modularity for bipartite networks (Hypothesis 1; Table 1) -------------------
prepare.modularity.analysis <- function (web, folder, iterations=1000) {
  # Observed
  if (file.exists(folder)){stop('folder exists. remove it first if you wish to re-analyze')}
  dir.create(folder)
  setwd(folder)
  web2edges(web, weight.column=F, out.files=c('edges', 'names', 'groups'))
  file.rename(from='web-names.lut', 'Node_names.txt')
  file.rename(from='web.groups', 'Node_groups.txt')
  file.rename(from='web.pairs', 'bipartite_binary_observed.txt')
  file.copy('~/Dropbox/PROJECTS/Functions_and_algorithms/Modularity_bipartite_binary/.', to=getwd(), recursive=T, overwrite=T, copy.mode=T)
  # Randomized
  dir.create('bipartite_binary_prob')
  setwd(paste(folder,'bipartite_binary_prob',sep='/'))
  web.rand <- prob_model(web, iter=iterations)
  x <- 0
  web.edgelist.rand <- list()
  # Write each randomized edgelist to a file to later be read by the modularity algorithm
  for (w in web.rand){
    x <- x+1
    web.edgelist.rand[[x]] <- web2edges(w, weight.column=F, return=T)
    write.table(web.edgelist.rand[[x]], paste(x,'.txt',sep=''), sep=' ', row.names=F, col.names=F, quote=F)
  }
}


modularity.results <- function (folder, AtrribTable, iterations=1000) {
  setwd(folder)
  #   mod.results <- list()
  heuristics <- read.table('bipartite_binary_observed_modularity.txt', header=F, col.names='M.huer') # Select the maximum value from the heuristic runs of the real network
  (M.huer.max <- max(heuristics$M.huer))
  # See how many of the randomized webs are greater than the observed M
  M.randomizations <- read.table('bipartite_binary_prob/mod_rand.txt', header=F, col.names='M.rand')
  (pval <- sum(M.randomizations$M.rand > M.huer.max)/iterations)
  if (pval == 0){pval <- 0.001}
  #   mod.results[[pval]] <- pval
  
  ### Extract information on module composition
  file.modules <- paste('modules-',which(heuristics$M.huer == max(heuristics$M.huer))[1],'-bipartite_binary_observed.txt',sep='')
  Nodes <- read.table('Node_names.txt', he=T) # Read the node IDs from the file
  Nodes$Module <- NA
  # Read the modularity file to make a list of modules and their nodes
  Modules <- read.table(file.modules, sep=' ', fill=T, he=F)
  if (length(Modules[,10:length(Modules)]) < max(Modules$V2, na.rm=T)) {
    stop('something wrong in file modules file. Try to enter spaces manually at the end of lines')
  }
  
  ModuleList <- list() # Extract information on module affiliation from the file
  for (i in 1:length(Modules$V1)){
    tmp <- unmatrix(Modules[i,], byrow=T) #unmatrix is from the gdata package
    names(tmp) <- NULL
    tmp <- na.omit(tmp)
    tmp <- as.numeric(tmp[(which(tmp == "---")+1):length(tmp)]) # the index numbers of the nodes in that module
    ModuleList[[i]]  <- sort(tmp)
    # Assign each node to its module
    Nodes[which(Nodes$virtual %in% tmp), 'Module'] = i
  }
  #   mod.results[[Nodes]] <- Nodes
  
  # Write the module affiliation to the ttribute table
  AtrribTable$Module <- NA
  AtrribTable$Module <- Nodes$Module[match(AtrribTable$RodentID, Nodes$real)]
  # print(AtrribTable[1:10,c('RodentID', 'NodeID', 'Module', 'Bart.infect')])
  #   Nodes$real[1:length(AtrribTable$RodentID)] == AtrribTable$RodentID
  
  print('P value')
  print(pval)
  print('Module identity')
  print(sort(unique(AtrribTable$Module))) # Module identities
  return(list(M=M.huer.max,Pval=pval,Modules=unique(Nodes$Module),AtrribTable=AtrribTable))
}


### Prepare for analyses. Only analyze networks with > 10 individuals and > 3 parasites
# Buriram
prepare.modularity.analysis(M.Bur,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/BUR')
prepare.modularity.analysis(M.Bur.Bs,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/BUR/Bansav')
# Mondolkiri
prepare.modularity.analysis(M.Mon,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/MON/')
prepare.modularity.analysis(M.Mon.Bs,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/MON/Bansav/')
prepare.modularity.analysis(M.Mon.Rt,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/MON/Rattan/')
# Sihanouk
prepare.modularity.analysis(M.Sih,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/SIH/')
prepare.modularity.analysis(M.Sih.Re,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/SIH/Ratexu/')
prepare.modularity.analysis(M.Sih.Rt,'~/Dropbox/PROJECTS/REV_Pilosof_2014/Manuscript/PLoS One/Analysis/SIH/Rattan/')

### Modularity analysis can be run from here, as in the following two lines. But it is faster to open parallel terminals
### and run from there for several webs at a time.
system('./bipartite_binary_obs.sh')        
system('./bipartite_binary_prob.sh')

### Collect and organize results after running the analyses with Bash in Linux
Modularity.results <- empty.df(c('M','Pval','Modules'))

### BURIRAM
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/BUR/', web.Bur$attrib, iterations = 1000)
Modularity.results['Bur','M'] <- tmp$M
Modularity.results['Bur','Pval'] <- tmp$Pval
Modularity.results['Bur','Modules'] <- length(table(tmp$Modules))
web.Bur$attrib <- tmp$AtrribTable
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/BUR/Bansav/', web.Bur.Bs$attrib, iterations = 1000)
Modularity.results['Bur.Bs','M'] <- tmp$M
Modularity.results['Bur.Bs','Pval'] <- tmp$Pval
Modularity.results['Bur.Bs','Modules'] <- length(table(tmp$Modules))
web.Bur.Bs$attrib <- tmp$AtrribTable
### MONDOLKIRI
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/MON/', web.Mon$attrib, iterations = 1000)
Modularity.results['Mon','M'] <- tmp$M
Modularity.results['Mon','Pval'] <- tmp$Pval
Modularity.results['Mon','Modules'] <- length(table(tmp$Modules))
web.Mon$attrib <- tmp$AtrribTable
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/MON/Bansav/', web.Mon.Bs$attrib, iterations = 1000)
Modularity.results['Mon.Bs','M'] <- tmp$M
Modularity.results['Mon.Bs','Pval'] <- tmp$Pval
Modularity.results['Mon.Bs','Modules'] <- length(table(tmp$Modules))
web.Mon.Bs$attrib <- tmp$AtrribTable
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/MON/Rattan/',web.Mon.Rt$attrib, iterations = 1000)
Modularity.results['Mon.Rt','M'] <- tmp$M
Modularity.results['Mon.Rt','Pval'] <- tmp$Pval
Modularity.results['Mon.Rt','Modules'] <- length(table(tmp$Modules))
web.Mon.Rt$attrib <- tmp$AtrribTable
### SIHANOUK
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/SIH', web.Sih$attrib, iterations = 1000)
Modularity.results['Sih','M'] <- tmp$M
Modularity.results['Sih','Pval'] <- tmp$Pval
Modularity.results['Sih','Modules'] <- length(table(tmp$Modules))
web.Sih$attrib <- tmp$AtrribTable
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/SIH/Ratexu/', web.Sih.Re$attrib, iterations = 1000)
Modularity.results['Sih.Re','M'] <- tmp$M
Modularity.results['Sih.Re','Pval'] <- tmp$Pval
Modularity.results['Sih.Re','Modules'] <- length(table(tmp$Modules))
web.Sih.Re$attrib <- tmp$AtrribTable
tmp <- modularity.results('~/Dropbox/PROJECTS/REV_Pilosof_2014/SIH/Rattan/', web.Sih.Rt$attrib, iterations = 1000)
Modularity.results['Sih.Rt','M'] <- tmp$M
Modularity.results['Sih.Rt','Pval'] <- tmp$Pval
Modularity.results['Sih.Rt','Modules'] <- length(table(tmp$Modules))
web.Sih.Rt$attrib <- tmp$AtrribTable


### Calculate moduarity when forcing the modules by taxonomy

web <- web.Sih
web$attrib$M.Taxon <- as.numeric(web$attrib$RodentSp)
Mb <- 0
for (s in 1:max(web$attrib$M.Taxon)){
  s.mat <- web$bipartite[rownames(web$bipartite) %in% web$attrib$RodentID[web$attrib$M.Taxon==s],]
  s.mat <- bipartite::empty(s.mat)
  cij=as.dist(tcrossprod(s.mat))
  sum_cij=sum(cij)
  sum_ma.1 <- sum_ma.2 <- 0
  for (a in colnames(s.mat)){
    ma <- sum(s.mat[,a])
    sum_ma.1 <- sum_ma.1+ma*(ma-1)
    sum_ma.2 <- sum_ma.2+ma
  }
  sum_ma.2 <- sum_ma.2^2
  
  t_it_j <- rowSums(s.mat)
  t_it_j <- combn(t_it_j,2)
  sum_titj <- sum(t_it_j[1,]*t_it_j[2,])
  Mb <- Mb+sum_cij/sum_ma.1-sum_titj/sum_ma.2
}
Mb


################################# Matrices for MRM (Hypothesis 1; Fig. 2) -------------------
build.matrix.variable <- function(dataset, trait, type){
  if (!type %in% c('discrete', 'cont')){stop("type must be 'discrete' or 'cont'")}
  
  node.count <- length(dataset$NodeID)
  M.expl <- matrix(nrow=node.count, ncol=node.count, dimnames=list(sort(dataset$RodentID), sort(dataset$RodentID)))
  M.expl[,] <- 0
  # For discrete variables:
  if (type=='discrete'){
    trait.split <- split(dataset$RodentID, factor(get(trait,dataset)))
    for (sp in trait.split){
      if (length(sp) == 1) {print('TRAIT LEVEL SKIPPED:');
                            print(names(unlist(trait.split))[which(unlist(trait.split) %in% sp == T)]);
                            print(sp);
                            next} # Skip when there is only 1 individual with a certain trait
      pairwise <- combn(sort(sp), 2) # Find all pairwise combinations of nodes
      for (i in 1:dim(pairwise)[[2]]){
        M.expl[pairwise[2,i], pairwise[1,i]] <- 1 # Fill the lower triangle of the response matrix
      }
    }
  }
  # For continuous variables
  if (type=='cont'){
    for (i in 1:dim(M.expl)[1]){
      for (j in i:dim(M.expl)[2]){
        M.expl[j,i] <- abs(dataset[dataset$RodentID==rownames(M.expl)[i],trait]-dataset[dataset$RodentID==rownames(M.expl)[j],trait])
      }
    }
  }
  
  # Check there's nothing in the upper trianlge or diagonal
  print(paste(trait, sum(M.expl[upper.tri(M.expl)]), sum(diag(M.expl)), dim(M.expl)[1], sep=' | '))
  if (var(M.expl[lower.tri(M.expl)], na.rm=T) == 0){print('All values are the same, there is no variation in the variable')}
  return(M.expl)
}

# The dataset
prepare.MRM <- function (dataset, multispecies=F, rm.na=F, scale=T) {
  # Build matrices
  matrix.list <- list()
  Y <- build.matrix.variable(dataset, 'Module', 'discrete')
  X_age <- build.matrix.variable(dataset, 'Age', 'discrete')
  X_habitat <- build.matrix.variable(dataset, 'Habitat', 'discrete')
  X_sex <- build.matrix.variable(dataset, 'Sex', 'discrete')
  X_RSM <- build.matrix.variable(dataset, 'RSM', 'cont')
  X_BM <- build.matrix.variable(dataset, 'BM', 'cont')
  # Remove NA values
  if (rm.na==T){
    rm.na.mt <- function (mat, NA.mat){
      mat <- mat[rownames(NA.mat), colnames(NA.mat)]
      if (var(mat[lower.tri(mat)], na.rm=T) == 0){print('All values are the same, there is no variation in the variable')}
      return(mat)
    }
    X_RSM <- bipartite::empty(X_RSM)
    Y <- rm.na.mt(Y, X_RSM)
    X_BM <- rm.na.mt(X_BM, X_RSM)
    X_habitat <- rm.na.mt(X_habitat, X_RSM)
    X_sex <- rm.na.mt(X_sex, X_RSM)
    X_age <- rm.na.mt(X_age, X_RSM)
    
  }
  
  variation <- function (mat){
    if (var(mat[lower.tri(mat)], na.rm=T) == 0) {
      return(FALSE)}
    else {
      return(TRUE)
    }
  }

  matrix.list$X_BM <- X_BM
  matrix.list$X_RSM <- X_RSM
  if (variation(X_habitat)){matrix.list$X_habitat <- X_habitat}
  if (variation(X_sex)){matrix.list$X_sex <- X_sex}
  if (variation(X_age)){matrix.list$X_age <- X_age}
  if (variation(Y)){matrix.list$Y <- Y}
   
  # For multi-species: Patristic distance among individuals   
  if (multispecies){
    (node.count <- length(dataset$NodeID))
    X_patristic <- matrix(nrow=node.count, ncol=node.count, dimnames=list(sort(dataset$RodentID), sort(dataset$RodentID)))
    X_patristic[,] <- 0
    pairwise <- combn(sort(dataset$RodentID), 2) # Find all pairwise combinations of nodes
    for (i in 1:dim(pairwise)[[2]]){
      sp.i <- as.character(dataset[dataset$RodentID == pairwise[1,i], 'RodentSp'])
      sp.j <- as.character(dataset[dataset$RodentID == pairwise[2,i], 'RodentSp'])
      X_patristic[pairwise[2,i], pairwise[1,i]] <- cophenetic(MyTree)[sp.i,sp.j]
    }
    sum(X_patristic[upper.tri(X_patristic)]) # Check there's nothing in the upper trianlge
    sum(diag(X_patristic))
    X_patristic <- X_patristic[rownames(X_RSM), colnames(X_RSM)]
    matrix.list$X_patristic <- X_patristic
  }

  if (scale){
    Y <- matrix.list$Y
    matrix.list <- lapply(matrix.list, function (x) if (all(x %in% 0:1)) {as.dist(x)} else {as.dist(scale(x))})
    matrix.list$Y <- as.dist(Y)
  }
  else {
    matrix.list <- lapply(matrix.list, function (x) as.dist(x))
  }
return(matrix.list)
}

MRM.Bur <- prepare.MRM(web.Bur$attrib, multispecies=T, rm.na=T)
MRM.Mon <- prepare.MRM(web.Mon$attrib, multispecies=T)  # Do not test for RSM in Mondolkiri - too many missing cases
MRM.Mon.Bs <- prepare.MRM(web.Mon.Bs$attrib)  # Do not test for RSM in Mondolkiri - too many missing cases
MRM.Mon.Rt <- prepare.MRM(web.Mon.Rt$attrib)  # Do not test for RSM in Mondolkiri - too many missing cases
MRM.Sih <- prepare.MRM(web.Sih$attrib, multispecies=T, rm.na=T)
MRM.Sih.Re <- prepare.MRM(web.Sih.Re$attrib)
MRM.Sih.Rt <- prepare.MRM(web.Sih.Rt$attrib, rm.na=T)

summarize.results.modularity <- function (Loc, Sp, coeff){
  data.frame(Locality=rep(Loc, length(coeff[,1])), Species=rep(Sp, length(coeff[,1])), Trait=rownames(coeff), b=round(coeff[,1],3), pval=round(coeff[,2],3), stringsAsFactors=T, row.names=NULL)
}

s <- with(MRM.Bur, MRMlogit(Y~X_patristic+X_BM+X_RSM+X_habitat+X_sex+X_age))$coef
MRM.results <- summarize.results.modularity('Bur', 'All', s[-1,])
s <- with(MRM.Mon, MRMlogit(Y~X_patristic+X_BM+X_habitat+X_sex+X_age))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Mon', 'All', s[-1,]))
s <- with(MRM.Mon.Bs, MRMlogit(Y~X_BM+X_habitat+X_sex+X_age))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Mon', 'Bs', s[-1,]))
s <- with(MRM.Mon.Rt, MRMlogit(Y~X_BM+X_habitat+X_sex+X_age))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Mon', 'Rt', s[-1,]))
s <- with(MRM.Sih, MRMlogit(Y~X_patristic+X_BM+X_RSM+X_habitat+X_sex+X_age))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Sih', 'All', s[-1,]))
s <- with(MRM.Sih.Re, MRMlogit(Y~X_BM+X_RSM+X_sex))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Sih', 'Re', s[-1,]))
s <- with(MRM.Sih.Rt, MRMlogit(Y~X_BM+X_RSM+X_habitat+X_sex+X_age))$coef
MRM.results <- rbind(MRM.results, summarize.results.modularity('Sih', 'Rt', s[-1,]))

MRM.results$Signif <- FALSE
MRM.results[MRM.results$pval<0.05, 'Signif'] <- T # Significance according to the standradized coefficients


################################# Centrality - Linear model (Hypothesis 2; Fig. 2)  -------------------
setwd(MyWorkingFolder)

# This function runs step-wise regression models for centrality as function of traits and also multi-model inference process.
run.models.centrality <- function(web, Global='EC ~ RodentSp+BM+RSM+Habitat+Sex+Age', rm.na=F, normality=F, verbose=F, record.models=F){
  # Calculate EC
  # Use <<- so that attrib will be a global variable. this is crucial for a proper functioning of the stepAICc function
  attrib <<- web$attrib
  if (rm.na){attrib <<- attrib[complete.cases(attrib),]}
  # Global model
  m.global <- lm(Global, data=attrib)
  # Stepwise regression
  if (verbose){m.best <- stepAICc(m.global)} 
  else {
    sink('tmp.txt')
    m.best <- stepAICc(m.global)
    sink()
    unlink('tmp.txt')
  }
  # Null model
  m.null <- lm(EC ~ 1, data=attrib)
  # Create a list with the 'discarded' models to consider in multi-model inference
  models <- list()
  m <- m.global
  for (i in 2:length(m.best$anova$Step)){
    m <- update(m, formula(paste('~.-',str_sub(m.best$anova[i,1],3,10)),sep=''))
    models[[i-1]] <- m
  }
  # Multi-model inference
  if (length(m.best$coefficients) == 1){modlist <-c(models, list(m.global)); model.selec <- model.avg(modlist)} #if best model is a null model
  else {
    modlist <- c(models, list(m.global), list(m.null))
    model.selec <- model.avg(modlist)
  }
  if (normality) {sapply(c(models, list(m.global)), normality.assumptions)}
  if (record.models){
    modtable <- round(summary(model.selec)$summary,3)
    rows <- rownames(modtable)
    for (r in rows){
      term=NULL
      ch.terms <- substring(r, seq(1,nchar(r),1), seq(1,nchar(r),1))
      if ((ch.terms)[1]=="("){next}
      for (i in 1:length(ch.terms)){term=paste(term, names(model.selec$term.codes)[as.numeric(ch.terms[i])],sep='+')}
      rows[which(rows==r)]  <- term
    }
    colnames(modtable)[1] <- substr(deparse(substitute(web)),5,10)
    rownames(modtable) <- unlist(lapply(rows, function (x) str_replace(x,"+","EC~")))
    write.table(modtable, 'Multimodel_Inference.csv', append=T, sep=',')
  }
  if (verbose) {summary(model.selec)}
  return(summary(model.selec))
}


create.results.centrality <- function (Loc, Sp, coeff){
  if (is.null(names(coeff))){stop('no rownames reprsenting traits')}
  data.frame(Locality=rep(Loc, length(coeff)), Species=rep(Sp, length(coeff)), Trait=names(coeff), I=round(coeff,3), stringsAsFactors=T, row.names=NULL)
}


#### BURIRAM
s <- run.models.centrality(web.Bur, verbose=T, record.models=F)
results.centrality <- create.results.centrality('BUR', 'All', s$importance)
s <- run.models.centrality(web.Bur.Bs, Global='EC ~ BM+Sex+Age', verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('BUR', 'Bs', s$importance))
#### Mondokiri
s <- run.models.centrality(web.Mon, Global='EC ~ RodentSp+BM+Habitat+Sex+Age', verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('MON', 'All', s$importance))
s <- run.models.centrality(web.Mon.Bs, Global='EC ~ BM+Habitat+Sex+Age', verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('MON', 'Bs', s$importance))
s <- run.models.centrality(web.Mon.Rt, Global='EC ~ BM+Habitat+Sex+Age', verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('MON', 'Rt', s$importance))
#### SIHANOUK
s <- run.models.centrality(web.Sih, rm.na=T, verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('SIH', 'All', s$importance))
s <- run.models.centrality(web.Sih.Re, Global='EC ~ BM+RSM+Sex', verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('SIH', 'Re', s$importance))
s <- run.models.centrality(web.Sih.Rt, Global='EC ~ BM+RSM+Habitat+Sex', rm.na=T, verbose=T, record.models=F)
results.centrality <- rbind(results.centrality, create.results.centrality('SIH', 'Rt', s$importance))

# Plot results for modularity and centrality (Fig 2)
results.combined <- rbind(MRM.results[,1:3], results.centrality[,1:3])
results.combined$Locality <- toupper(results.combined$Locality)
results.combined$Trait <- revalue(results.combined$Trait, c("X_patristic"="Phylogeny", "RodentSp"="Phylogeny"))
results.combined$Trait <- revalue(results.combined$Trait, c("X_BM"="BM", "X_RSM"="RSM", "X_habitat"="Habitat", "X_sex"="Sex","X_age"="Age"))
results.combined$Trait <- factor(results.combined$Trait, c('Phylogeny','BM', 'RSM', 'Habitat', 'Sex', 'Age'))
results.combined$Value <- c(MRM.results$b, results.centrality$I)
results.combined$ValueType <- c(rep('Modularity',length(MRM.results$b)), rep('Centrality', length(results.centrality$I)))
results.combined$ValueType <- as.factor(results.combined$ValueType)
results.combined$ValueType <- relevel(results.combined$ValueType, 'Modularity')
results.combined$Signif <- c(MRM.results$Signif, rep(F, length(results.centrality$I)))

setwd('MyWorkingFolder/Manuscript/Figures/')
svg('Figure 2.svg', width=10, height=10)
X11()
p <- ggplot(results.combined, aes(x=Trait,y=Value, shape=Signif, colour=Species))
p+geom_point(size=5)+
  facet_grid(ValueType~Locality, scales='free_y')+theme_bw()+
  theme(text = element_text(size=20), axis.text.x = element_text(angle = -90, hjust=0), legend.position='right')
dev.off()


################################# Centrality - correlations (Hypothesis 2; Fig. 3)-------------------

# This function correlates the position of individuals in the multi- vs. single-species network
create.res.corr <- function(Locality, Species, plot.it=F){
  d.all <- get(paste('web',Locality,sep='.'))$attrib
  d.species <- get(paste('web',Locality,Species,sep='.'))$attrib
  tmp <- merge(d.species, d.all, by='RodentID')
  if (plot.it){
    plot(tmp$EC.x, tmp$EC.y)
    text(tmp$EC.x, tmp$EC.y, tmp$RodentID)
  }
  r <- rcorr(tmp$EC.x, tmp$EC.y, type='pearson')
  return(data.frame(Locality=Locality, Species=Species, r=round(r$r[2],3), n=r$n[2], Pval=round(r$P[2],2)))
}

results.centrality.corr <- create.res.corr('Bur', 'Bs', T)
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Bur', 'Mc', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Bur', 'Re', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Mon', 'Bs', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Mon', 'Rt', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Sih', 'Ra', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Sih', 'Re', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Sih', 'Rn', T))
results.centrality.corr <- rbind(results.centrality.corr, create.res.corr('Sih', 'Rt', T))

results.centrality.corr$Signif <- FALSE
results.centrality.corr[results.centrality.corr$Pval<0.05, 'Signif'] <- TRUE # Significance according to the standradized coefficients
results.centrality.corr$Species <- factor(results.centrality.corr$Species, c('Bs', 'Rt', 'Re', 'Ra', 'Rn', 'Mc'))

svg('Figure 3_20141013.svg', width=10, height=10)
p <- ggplot(results.centrality.corr, aes(x=Species, y=r, shape=Signif))

p+geom_point(aes(colour=Locality),  size=6)+
  theme_bw()+scale_colour_brewer(palette='Dark2')+geom_abline(intercept=0, slope=0, lty=2)+
  theme(text = element_text(size=20), panel.grid.major=element_blank(), panel.grid.minor=element_blank())+
  labs(x='Species', y='Correlation coefficient')

dev.off()

# Plot an example for a correlation (the inset of Fig 3)
svg('Figure 3 inset_20141013.svg', width=5, height=5)
tmp <- merge(web.Sih.Rt$attrib, web.Sih$attrib, by='RodentID')
p <- ggplot(tmp, aes(x=tmp$EC.x, y=tmp$EC.y, label=tmp$RodentID))
p+geom_point(size=6, colour='blue')+stat_smooth(method="lm", se=F)+
  geom_text(position = position_jitter(w = 0.05, h = 0.05), size=4)+
  theme_bw()+
  theme(text = element_text(size=14), panel.grid.major=element_blank(), panel.grid.minor=element_blank())+labs(x='Centrality in single-species network', y='Centrality in multi-species network')
dev.off()


################################# Simulations (Fig. 4)  -------------
# Sub-sample networks of size N from source.web. The networks are multi-species
create.subnetworks.multispecies <- function(source.web, N, n.networks=250){
  subnetworks <- list()                                                             # A list to store the random networks
  n.species.prop <- table(V(source.web)$Species)/sum(table(V(source.web)$Species))  # Keep proportion of species in the subweb
  i <- 1                                                                            # First time step
  while (i <= n.networks){
    # Select N random individuals to remove while keeping the proportion of individuals from different speceis
    random.individuals <- sample(V(source.web)$name, N, replace=F, prob=n.species.prop[match(V(source.web)$Species, names(n.species.prop))])
    sub.web <- induced.subgraph(graph=source.web, vids=random.individuals,  impl='copy_and_delete') # Extract a random network
    # Only keep subwebs which have a higher connectance than the source web. This is because later on edges can be
    # removed to match the connectance to that of the source web.
    if (graph.density(sub.web)>graph.density(source.web)){
      # Remove edges so that the subweb will have the same density (connectance) as the source multi-species web. But do
      # not delete edges of vertices with degre 1 to not fragment the web. Connectance is calclated as E/((S^2-S)/2) 
      # where E is the number of realized edges and S is the number of vertices. the -S is to ignore diagonal of
      # adjacency matrix.
      number.edges.to.remove <- ecount(sub.web)-round(graph.density(source.web)*((vcount(sub.web)^2-vcount(sub.web))/2))
      vertices <- V(sub.web)[which(degree(sub.web)>1)]
      sub.web <- delete.edges(sub.web, E(sub.web)[from(vertices)][sample(ecount(sub.web),number.edges.to.remove)])
      # Discard networks which have more than one component.
      if (clusters(sub.web)$no == 1){
        subnetworks[[i]] <- sub.web
        i <- i+1
      }
    }
  }
  return(subnetworks)
}

# Subsample networks of single species from the source network. The connectance parameter determines the desired
# connectance of the subweb
create.subnetworks.singlespecies <- function(source.web, C, n.networks=250){
  subwebs <- list() # A list to store the networks
  # calculate the number of edges to remove so that subwebs will have the same connectance (or at least proximate) as C 
  number.edges.to.remove <- ecount(source.web)-round(C*((vcount(source.web)^2-vcount(source.web))/2))
  i <- 1
  while (i <= n.networks){
    subwebs[[i]] <- delete.edges(source.web, E(source.web)[sample(ecount(source.web),number.edges.to.remove)])
    # If  there are vertices with degree 0 discard the subweb. So proceed for subwes where all vertices have at
    # least one edge.
    if (clusters(subwebs[[i]])$no == 1){i <- i+1}
  }
  return(subwebs)
}

# This function calculates the transmission rate theta between species m and k based on the similarities between them in
# terms of shared parasites. See the methods section of the paper for details.
calculate.theta.multispecies <- function(theta, web){
  M.species <- web$bipartite
  A.species <- web$attrib
  rownames(M.species) <- A.species$RodentSp[match(rownames(M.species), A.species$RodentID)]
  M.species <- as.data.frame(M.species)
  M.species$RodentSp <- row.names(M.species)
  M.species <- aggregate(.~RodentSp, data=M.species, sum) # The number of individuals of a certain host infected by a certain parasite
  rows <- M.species$RodentSp
  M.species <- as.matrix(M.species[,2:length(M.species)])
  rownames(M.species) <- rows
  beta.mk <- vegdist(M.species, method='jaccard', binary=T)
  theta.mk <- theta*(1-as.matrix(beta.mk))
  return(theta.mk)
}


# This function simulates the spread of a novel parasite. theta can be either fixed or a matrix in the case of
# multi-species TPNs. See details on simulation in the paper.
simulate.parasite.spread <- function(theta, g, n=250){
  res <- NULL
  node.count <- length(V(g))
  if (node.count<10){stop('Less than 10 individuals')}
  # Determine if the network has multiple species
  if (length(unique(V(g)$Species))>1){multi=T
  } else {
    multi=F
  }
  for (iter in 1:n){           # Start iterations
    nodes.state <- rep('S',node.count)               # Create a vector of SI states
    nodes.state[sample(1:node.count, 1)] <- 'I'      # Randomly select a node to start with
    node.count.I <- 1                                # Initial count of infected individuals
    time.steps <- 0                                  # Initialize the time steps it takes from begining to full infection
    # While there are still susceptible individuals
    while (node.count.I < node.count){               
      time.steps <- time.steps+1        # increase time step
      nodes.I <- which(nodes.state == 'I')  # Infected nodes for this time step. This has to be fixed at this point because nodes are getting infected as the loop runs
      for (i in nodes.I){                                                       # For each infected node i...
        S.neighbors <- neighborhood(g, order=1, nodes=i)[[1]][-1]               # The neighbors of the infected node i
        S.neighbors <- S.neighbors[which(nodes.state[S.neighbors] == 'S')]      # Select only the susceptible neighbors of node i
        for (s in S.neighbors){                                                 # For each susceptible neighbor
          # If it is a multi-species network take theta from the matrix. Otherwise, take the fixed theta.
          if (is.matrix(theta)){th <- theta[V(g)$Species[i], V(g)$Species[s]]
          } else {
            th <- theta
          }
          if ((1-(1-th)^(E(g)[to(i) & from(s)]$weight)) >= runif(1,0,1)){ # If the probability of infection is greater than a random number then infect s
            nodes.state[s] <- 'I'                                         # change status of s to Infeted
            node.count.I <- node.count.I+1                                # increase the count of infected nodes.
          }  # End infection of node s by node i
        } # End the infection process of all S neighbors of i
      } # End the infection process for all infected nodes for this time step
    } # End infection for all nodes in the network
    res[iter] <- time.steps # Write the number of time steps it took to infect all individuals in this iteration
    print(paste('Multi: ',multi, ' | nodes: ',node.count, ' | Iteration: ',iter,'/',n, ' | TGI: ',time.steps,' | ',format(Sys.time(), "%Y-%m-%d %H:%M:%S"), sep=''))
  } # End of iterations
  return(res) # Return the vector of TGI
}

# This is the function that manages and runs the simulations
run.simulation <- function(multi.sp.web, single.sp.web, n.subwebs=250, iterations=250, theta=0.02){
  
  if (vcount(single.sp.web$g) < 10){stop('Less than 10 individuals')}
  
  file.multi.thetamk <- paste('res_sim_',theta,'_multinotfixed_',substr(deparse(substitute(single.sp.web)),5,10),'.csv',sep='')
  file.multi.theta <- paste('res_sim_',theta,'_multifixed_',substr(deparse(substitute(single.sp.web)),5,10),'.csv',sep='')
  file.single <- paste('res_sim_',theta,'_single_',substr(deparse(substitute(single.sp.web)),5,10),'.csv',sep='')
  
  n.start.m <- n.start.s <- 0
  if (file.exists(file.multi.thetamk)){n.start.m <- length(readLines(file.multi.thetamk))}
  if (file.exists(file.single)){n.start.s <- length(readLines(file.single))}
    
  ## Step 1: create subwebs
  
  # Create a list of 250 multi-species sub-networks with the same number of nodes as the single-species network to which
  # it is compared and connectance proximate to that of the original multi-species network
  subwebs.multi <- create.subnetworks.multispecies(multi.sp.web$g, vcount(single.sp.web$g), n.networks=n.subwebs)
  # Create a list of 250 single-species sub-networks with the same connectance as that of the original multi-species
  # network
  subwebs.single <- create.subnetworks.singlespecies(single.sp.web$g, graph.density(multi.sp.web$g), n.networks=n.subwebs)
  
  ## Step 2: run simulation for the multi-species sub-networks
  results.simulation.multi <- results.simulation.multi.thetamk <- matrix(data = NA,
                                                                         nrow = n.subwebs,
                                                                         ncol = iterations,
                                                                         dimnames=list(paste('SW',1:n.subwebs,sep=''),row.names(1:iterations)))
  theta.mk <- calculate.theta.multispecies(theta, multi.sp.web)
  
  if (file.exists(file.multi.theta)){
    tmp <- read.table(file.multi.thetamk, sep=',', header = F)
    results.simulation.multi.thetamk[1:n.start.m,] <- as.matrix(tmp[,2:length(tmp)])
    tmp <- read.table(file.multi.theta, sep=',', header = F)
    results.simulation.multi[1:n.start.m,] <- as.matrix(tmp[,2:length(tmp)])
  }
  for (sw in (n.start.m+1):n.subwebs){
#     plot(subwebs.multi[[sw]],
#          layout=layout.kamada.kawai,
#          vertex.color="purple",
#          edge.width=E(subwebs.multi[[sw]])$weight,
#          main=paste(unique(multi.sp.web$attrib$Locality), 'multi',sw))
    print(paste('MULTI-species sub-networks -- ',sw,' / ',n.subwebs,sep=''))
    results.simulation.multi.thetamk[sw,] <- simulate.parasite.spread(theta.mk, g=subwebs.multi[[sw]], n=iterations)
    results.simulation.multi[sw,] <- simulate.parasite.spread(theta, g=subwebs.multi[[sw]], n=iterations)
    write.table(results.simulation.multi.thetamk[1:sw,], file.multi.thetamk,
                append = F, row.names = T, col.names = F, sep=',')
    write.table(results.simulation.multi[1:sw,], file.multi.theta,
                append = F, row.names = T, col.names = F, sep=',')
  }
  
  ## Step 3: run simulation for the single-species sub-networks
  results.simulation.single <- matrix(data = NA,
                                      nrow = n.subwebs,
                                      ncol = iterations,
                                      dimnames=list(paste('SW',1:n.subwebs,sep=''),row.names(1:iterations)))
  if (file.exists(file.single)){
    tmp <- read.table(file.single, sep=',', header = F)
    results.simulation.single[1:n.start.s,] <- as.matrix(tmp[,2:length(tmp)])
  }  
  for (sw in (n.start.s+1):n.subwebs){
#     plot(subwebs.single[[sw]],
#          layout=layout.kamada.kawai,
#          vertex.color="green",
#          edge.width=E(subwebs.single[[sw]])$weight,
#          main=paste(unique(single.sp.web$attrib$Locality), unique(single.sp.web$attrib$RodentSp),sw))
    print(paste('SINGLE-species sub-networks -- ',sw,' / ',n.subwebs,sep=''))
    results.simulation.single[sw,] <- simulate.parasite.spread(theta, g=subwebs.single[[sw]], n=iterations)
    write.table(results.simulation.single[1:sw,], file.single,
                append = F, row.names = T, col.names = F, sep=',')
  }
  
  ## Step 4: Return results
  return(list(multi=results.simulation.multi.thetamk, multi.fixed.theta=results.simulation.multi, single=results.simulation.single))
}


# Explore theta = 0.05
sim.Bur.Bs.05 <- run.simulation(multi.sp.web=web.Bur, single.sp.web=web.Bur.Bs, n.subwebs=250, iterations=250, theta=0.05)
sim.Mon.Bs.05 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Bs, n.subwebs=250, iterations=250, theta=0.05)
sim.Mon.Rt.05 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Rt, n.subwebs=250, iterations=250, theta=0.05)
sim.Sih.Re.05 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Re, n.subwebs=250, iterations=250, theta=0.05)
sim.Sih.Rt.05 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Rt, n.subwebs=250, iterations=250, theta=0.05)

# Explore theta = 0.05
sim.Bur.Bs.01 <- run.simulation(multi.sp.web=web.Bur, single.sp.web=web.Bur.Bs, n.subwebs=250, iterations=250, theta=0.01)
sim.Mon.Bs.01 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Bs, n.subwebs=250, iterations=250, theta=0.01)
sim.Mon.Rt.01 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Rt, n.subwebs=250, iterations=250, theta=0.01)
sim.Sih.Re.01 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Re, n.subwebs=250, iterations=250, theta=0.01)
sim.Sih.Rt.01 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Rt, n.subwebs=250, iterations=250, theta=0.01)

# Explore theta = 0.02
sim.Bur.Bs.02 <- run.simulation(multi.sp.web=web.Bur, single.sp.web=web.Bur.Bs, n.subwebs=250, iterations=250, theta=0.02)
sim.Mon.Bs.02 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Bs, n.subwebs=250, iterations=250, theta=0.02)
sim.Mon.Rt.02 <- run.simulation(multi.sp.web=web.Mon, single.sp.web=web.Mon.Rt, n.subwebs=250, iterations=250, theta=0.02)
sim.Sih.Re.02 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Re, n.subwebs=250, iterations=250, theta=0.02)
sim.Sih.Rt.02 <- run.simulation(multi.sp.web=web.Sih, single.sp.web=web.Sih.Rt, n.subwebs=250, iterations=250, theta=0.02)


create.panel.plot <- function (sim.data){
  res.ggplot <- data.frame(TGI=c(rowMeans(sim.data$single),rowMeans(sim.data$multi), rowMeans(sim.data$multi.mk)),
                           web=rep(c('Single','Multi', 'Multi.mk'),1, each=dim(sim.data[[1]])[1]))
  p <- ggplot(res.ggplot, aes(x=TGI)) +
    geom_density(data=subset(res.ggplot, web=='Single'), fill="red", alpha=0.5) + 
    geom_density(data=subset(res.ggplot, web=='Multi'), fill="blue", alpha=0.5) +
    geom_density(data=subset(res.ggplot, web=='Multi.mk'), fill="green", alpha=0.5) +
    theme_bw()+
    theme(panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())
  return(p)
}

read.sim.files <- function(f.multi.mk,f.multi,f.single){
  sim.multi <- read.table(f.multi, sep=',', header = F)
  rows <- sim.multi[,1]
  sim.multi <- as.matrix(sim.multi[,2:length(sim.multi)])
  rownames(sim.multi) <- rows
  
  sim.multi.mk <- read.table(f.multi.mk, sep=',', header = F)
  rows <- sim.multi.mk[,1]
  sim.multi.mk <- as.matrix(sim.multi.mk[,2:length(sim.multi.mk)])
  rownames(sim.multi.mk) <- rows
  
  sim.single <- read.table(f.single, sep=',', header = F)
  rows <- sim.single[,1]
  sim.single <- as.matrix(sim.single[,2:length(sim.single)])
  rownames(sim.single) <- rows
  
  return(list(single=sim.single, multi=sim.multi, multi.mk=sim.multi.mk))
}

overlap <- function(TGIs){
  require(sfsmisc)
  
  overlap.matrix <- matrix(data = NA, nrow = 3, ncol = 3, dimnames = list(c('s','f','n'),c('s','f','n')))
  
  s <- TGIs$TGI.single
  f <- TGIs$TGI.fixed
  n <- TGIs$TGI.notfixed
  
  ### define limits of a common grid, adding a buffer so that tails aren't cut off
  lower <- 0
  upper <- max(c(s,f,n)) + 1
  
  ### generate kernel densities
  ds <- density(s, from=lower, to=upper)
  df <- density(f, from=lower, to=upper)
  dn <- density(n, from=lower, to=upper)
  
  ### single vs. fixed
  d <- data.frame(x=ds$x, a=ds$y, b=df$y)
  # calculate intersection densities
  d$w <- pmin(d$a, d$b)
  # integrate areas under curves
  total <- integrate.xy(d$x, d$a) + integrate.xy(d$x, d$b)
  intersection <- integrate.xy(d$x, d$w)
  # compute overlap coefficient
  overlap.matrix['f','s'] <- round(2 * intersection / total,3)
  
  ### single vs. notfixed
  d <- data.frame(x=ds$x, a=ds$y, b=dn$y)
  # calculate intersection densities
  d$w <- pmin(d$a, d$b)
  # integrate areas under curves
  total <- integrate.xy(d$x, d$a) + integrate.xy(d$x, d$b)
  intersection <- integrate.xy(d$x, d$w)
  # compute overlap coefficient
  overlap.matrix['n','s'] <- round(2 * intersection / total, 3)
  
  ### not fixed vs. fixed
  d <- data.frame(x=dn$x, a=dn$y, b=df$y)
  # calculate intersection densities
  d$w <- pmin(d$a, d$b)
  # integrate areas under curves
  total <- integrate.xy(d$x, d$a) + integrate.xy(d$x, d$b)
  intersection <- integrate.xy(d$x, d$w)
  # compute overlap coefficient
  overlap.matrix['n','f'] <- round(2 * intersection / total, 3)
  
  return(overlap.matrix)
}

# Create Fig. 4
p1 <- create.panel.plot(read.sim.files('res_sim_0.02_multinotfixed_Bur.Bs.csv','res_sim_0.02_multifixed_Bur.Bs.csv','res_sim_0.02_single_Bur.Bs.csv'))
p2 <- create.panel.plot(read.sim.files('res_sim_0.02_multinotfixed_Mon.Bs.csv','res_sim_0.02_multifixed_Mon.Bs.csv','res_sim_0.02_single_Mon.Bs.csv'))
p3 <- create.panel.plot(read.sim.files('res_sim_0.02_multinotfixed_Mon.Rt.csv','res_sim_0.02_multifixed_Mon.Rt.csv','res_sim_0.02_single_Mon.Rt.csv'))
p4 <- create.panel.plot(read.sim.files('res_sim_0.02_multinotfixed_Sih.Re.csv','res_sim_0.02_multifixed_Sih.Re.csv','res_sim_0.02_single_Sih.Re.csv'))
p5 <- create.panel.plot(read.sim.files('res_sim_0.02_multinotfixed_Sih.Rt.csv','res_sim_0.02_multifixed_Sih.Rt.csv','res_sim_0.02_single_Sih.Rt.csv'))
# p6 is just a meaningless panel to later erase in InkScape
p6 <- qplot(Sepal.Length, Petal.Length, data = iris)+theme_bw()+theme(panel.grid.major = element_blank(),panel.grid.minor = element_blank())

svg('Figure 4_20141013.svg', width=13.6, height=7.8)
grid.arrange(p1, p2, p3,p4,p5,p6, ncol=3, nrow=2)
dev.off()


## Calculate and plot statistics.

# Kolmogorov-Smirnov test
sim.data <- read.sim.files('res_sim_0.02_multinotfixed_Bur.Bs.csv','res_sim_0.02_multifixed_Bur.Bs.csv','res_sim_0.02_single_Bur.Bs.csv')
sim.data <- read.sim.files('res_sim_0.02_multinotfixed_Mon.Bs.csv','res_sim_0.02_multifixed_Mon.Bs.csv','res_sim_0.02_single_Mon.Bs.csv')
sim.data <- read.sim.files('res_sim_0.02_multinotfixed_Mon.Rt.csv','res_sim_0.02_multifixed_Mon.Rt.csv','res_sim_0.02_single_Mon.Rt.csv')
sim.data <- read.sim.files('res_sim_0.02_multinotfixed_Sih.Re.csv','res_sim_0.02_multifixed_Sih.Re.csv','res_sim_0.02_single_Sih.Re.csv')
sim.data <- read.sim.files('res_sim_0.02_multinotfixed_Sih.Rt.csv','res_sim_0.02_multifixed_Sih.Rt.csv','res_sim_0.02_single_Sih.Rt.csv')

# Plot box-plots
TGIs <- data.frame(TGI=c(rowMeans(sim.data$single),rowMeans(sim.data$multi), rowMeans(sim.data$multi.mk)),
                   web=rep(c('Single','Multi', 'Multi.mk'),1, each=dim(sim.data[[1]])[1]))
TGIs$web <- factor(TGIs$web,c('Single', 'Multi', 'Multi.mk'))
svg('Figure 4_inset_5', width=7, height=7)
ggplot(TGIs, aes(x=TGIs$web, y=TGIs$TGI, fill=web))+geom_boxplot(notch=T, alpha=0.5)+stat_summary(fun.y=mean, colour="blue", geom="point")+
  theme_bw()+theme(legend.position="none",panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())+
  scale_fill_manual(values = c("red", "blue", "green"))+labs(title="")
dev.off()

TGIs <- data.frame(TGI.single=rowMeans(sim.data$single),
                         TGI.fixed=rowMeans(sim.data$multi),
                         TGI.notfixed=rowMeans(sim.data$multi.mk))
round((median(TGIs$TGI.fixed)-median(TGIs$TGI.single))/median(TGIs$TGI.single)*100,2)
round((median(TGIs$TGI.notfixed)-median(TGIs$TGI.single))/median(TGIs$TGI.single)*100,2)

round(abs(max(density(TGIs$TGI.fixed)$y)-max(density(TGIs$TGI.single)$y))/max(density(TGIs$TGI.single)$y)*100,2)
round(abs(max(density(TGIs$TGI.notfixed)$y)-max(density(TGIs$TGI.single)$y))/max(density(TGIs$TGI.single)$y)*100,2)


# (skewness(TGIs$TGI.fixed)-skewness(TGIs$TGI.single))/skewness(TGIs$TGI.single)
# (skewness(TGIs$TGI.notfixed)-skewness(TGIs$TGI.single))/skewness(TGIs$TGI.single)
# (skewness(TGIs$TGI.notfixed)-skewness(TGIs$TGI.fixed))/skewness(TGIs$TGI.fixed)

ks.test(density(TGIs$TGI.single)$y, density(TGIs$TGI.notfixed)$y)
ks.test(density(TGIs$TGI.single)$y, density(TGIs$TGI.fixed)$y)
ks.test(density(TGIs$TGI.fixed)$y, density(TGIs$TGI.notfixed)$y)


ks.test(TGIs$TGI.single, TGIs$TGI.notfixed)
ks.test(TGIs$TGI.single, TGIs$TGI.fixed)
ks.test(TGIs$TGI.fixed, TGIs$TGI.notfixed)

# wilcox.test(TGIs$TGI.single, TGIs$TGI.fixed)

overlap(TGIs)
svg('Figure 1_inset_AUC5', width=3.5, height=3.5)
o <- data.frame(comp=1:3, val=na.omit(as.vector(overlap(TGIs))))
ggplot(o, aes(x=o$comp,y=o$val))+geom_bar(stat='identity')+theme_bw()+theme(legend.position="none",panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())+ylim(0,1)
dev.off()


deviations <- data.frame(comp=c(rep('B-Bs',4),rep('M-Bs',4),rep('M-Rt',4),rep('S-Re',4),rep('S-Rt',4)),
                         var=rep(c('Med','Med','Max','Max'),5),
                         fixed=rep(c('Yes','No','Yes','No'),5),
                         dev=c(76.9,432.89,30.53,85.6,11.07,25.78,76.16,75.52,-3.54,14.04,55.98,66.59,29.86,125.3,31.03,75.1,17.15,91.95,33.23,74.93)
                         )         
svg('Figure 4_deviations.svg', width=3.5, height=3.5)
ggplot(deviations, aes(x=comp,y=dev,colour=fixed))+geom_point(size=6)+facet_grid(.~var, scales='free_y')+scale_color_manual(values=c('#ffcb00','#7f7fff'))+
  theme_bw()+theme(legend.position="none",panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())
dev.off()


ggplot(deviations[deviations$var=='Max',], aes(x=comp,y=dev,colour=fixed))+
  geom_point(size=6)+scale_color_manual(values=c('#ffcb00','#7f7fff'))+
  theme_bw()+theme(legend.position="none",panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())

svg('Figure 4_deviations.svg', width=3.5, height=3.5)
ggplot(deviations[deviations$var=='Med',], aes(x=comp,y=dev,colour=fixed))+
  geom_point(size=4)+scale_color_manual(values=c('#ffcb00','#7f7fff'))+
  theme_bw()+theme(legend.position="none",panel.grid.major=element_blank(),panel.grid.minor=element_blank(),axis.title.x=element_blank(),axis.title.y = element_blank())
dev.off()


##################### draw TPNs with Gephi ----------------

matrix.to.edgelist <- function(mat, mode=c('bipartite', 'unipartite')){
  if (is.null(rownames(mat)) | is.null(colnames(mat))) {stop('Either row or column names are null')}
  if (!mode %in% c('bipartite', 'unipartite')) {stop('please specify bipartite/unipartite mode')}
  
  if (mode=='bipartite'){
    rows <- dim(mat)[1]
    cols <- dim(mat)[2]
    EL <- empty.df(c('i','j','w'))
    x <- 0
    for (i in 1:rows){
      for (j in 1:cols){
        if (mat[i,j] > 0){
          x <- x+1
          EL[x,'i'] <- rownames(mat)[i]
          EL[x,'j'] <- colnames(mat)[j]
          EL[x,'w'] <- mat[i,j]
        }
      }
    }
  }
  
  if (mode=='unipartite'){
    if (!isSymmetric(mat)){ # If the matrix is not symmetric (directed)
      n <- 1
      EL  <- empty.df(c('i','j','w'))
      for (i in 1:dim(mat)[[1]]){
        for (j in 1:dim(mat)[[2]]){
          if (i != j){ # skip diagonal
            EL[n,'i'] <- rownames(mat)[i]
            EL[n,'j'] <- rownames(mat)[j]
            EL[n,'w'] <- mat[i,j]
            n <- n+1
          }
        }
      }
    } else { # If the matrix is symmetric (directed)
      n <- 1
      EL  <- empty.df(c('i','j','w'))
      for (i in 1:dim(mat)[[1]]){
        j <- 1 # Column counter
        while (i-j >= 1){ # Limit to lower diagonal
          if (mat[i,j] != 0){
            EL[n,'i'] <- rownames(mat)[i]
            EL[n,'j'] <- rownames(mat)[j]
            EL[n,'w'] <- mat[i,j]
            n <- n+1 # Increment row counter
          }
          j <- j+1 # move to next column
        }
      }
    }
  }
  return(EL)
}


create.gephi.uni <- function (web.uni, att.uni, filename) {
  
  gephi.attr <- data.frame(id=att.uni$NodeID, species=as.character(att.uni$RodentSp), module=att.uni$Module, EC=att.uni$EC, stringsAsFactors=F) 
  gephi.nodes <- data.frame(id=att.uni$NodeID, label=att.uni$RodentID, stringsAsFactors=F) 
  gephi.edges <- matrix.to.edgelist(mat=web.uni, mode='unipartite')
  gephi.edges$i <- gephi.nodes$id[match(gephi.edges$i, gephi.nodes$label)]
  gephi.edges$j <- gephi.nodes$id[match(gephi.edges$j, gephi.nodes$label)]
  colnames(gephi.edges) <- c('source', 'target', 'weight')
  write.gexf(nodes=gephi.nodes, edges=gephi.edges[,1:2], nodesAtt=gephi.attr, edgesWeight=gephi.edges[,3], output=filename)
}

create.gephi.uni(web.Sih$unip.betaD,web.Sih$attrib, 'SIH_multi.gexf')
create.gephi.uni(web.Sih.Rt$unip.betaD,web.Sih.Rt$attrib, 'SIH_Rt.gexf')
